@@ -1,13 +1,14 @@
-__version__ = "1.6.1"
+__version__ = "1.12.0"
 
 from copick.models import COPICK_TYPES
-from copick.ops.open import from_czcdp_datasets, from_file, from_string
+from copick.ops.open import from_czcdp_datasets, from_file, from_string, new_config
 
 __all__ = [
     "from_file",
     "from_string",
     "from_czcdp_datasets",
     "from_string",
+    "new_config",
     "__version__",
     "COPICK_TYPES",
 ]
