@@ -1,14 +1,18 @@
 import json
-from typing import Dict, Iterable, List, Literal, MutableMapping, Optional, Tuple, Type, Union
+from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Literal, MutableMapping, Optional, Tuple, Type, Union
 
 import numpy as np
-import trimesh
 import zarr
 from pydantic import AliasChoices, BaseModel, Field, field_validator
-from trimesh.parent import Geometry
 
 from copick.util.escape import sanitize_name
 from copick.util.ome import fits_in_memory, segmentation_pyramid, volume_pyramid, write_ome_zarr_3d
+from copick.util.relion import picks_to_df_relion, relion_df_to_picks
+
+# Don't import Geometry at runtime to keep CLI snappy
+if TYPE_CHECKING:
+    import pandas as pd
+    from trimesh.parent import Geometry
 
 
 class PickableObject(BaseModel):
@@ -24,6 +28,7 @@ class PickableObject(BaseModel):
         identifier: Identifier for the object (e.g. Gene Ontology ID or UniProtKB accession).
         map_threshold: Threshold to apply to the map when rendering the isosurface.
         radius: Radius of the particle, when displaying as a sphere.
+        metadata: Additional metadata for the object (user-defined contents).
     """
 
     name: str
@@ -35,6 +40,7 @@ class PickableObject(BaseModel):
     identifier: Optional[str] = Field(None, alias=AliasChoices("go_id", "identifier"))
     map_threshold: Optional[float] = None
     radius: Optional[float] = None
+    metadata: Optional[Dict[str, Any]] = Field(default_factory=dict)
 
     @property
     def go_id(self):
@@ -67,6 +73,11 @@ class PickableObject(BaseModel):
             raise ValueError(f"Name '{v}' contains invalid characters. Use copick.escape.sanitize_name() to clean it.")
         return v
 
+    @field_validator("metadata", mode="before")
+    @classmethod
+    def none_to_empty_dict(cls, v):
+        return {} if v is None else v
+
 
 class CopickConfig(BaseModel):
     """Configuration for a copick project. Defines the available objects, user_id and optionally an index for runs.
@@ -209,6 +220,7 @@ class CopickObject:
         pdb_id: PDB ID for the object.
         map_threshold: Threshold to apply to the map when rendering the isosurface.
         radius: Radius of the particle, when displaying as a sphere.
+        metadata: Additional metadata for the object (user-defined contents).
     """
 
     def __init__(self, root: "CopickRoot", meta: PickableObject):
@@ -228,10 +240,12 @@ class CopickObject:
         pdb_id = self.pdb_id if self.pdb_id is not None else "None"
         identifier = self.identifier if self.identifier is not None else "None"
         map_threshold = self.map_threshold if self.map_threshold is not None else "None"
+        metadata = self.metadata if self.metadata is not None else "None"
 
         ret = (
             f"CopickObject(name={self.name}, is_particle={self.is_particle}, label={label}, color={color}, "
-            f"emdb_id={emdb_id}, pdb_id={pdb_id}, identifier={identifier} threshold={map_threshold}) at {hex(id(self))}"
+            f"emdb_id={emdb_id}, pdb_id={pdb_id}, identifier={identifier} threshold={map_threshold}, "
+            f"metadata={metadata}) at {hex(id(self))}"
         )
         return ret
 
@@ -271,6 +285,10 @@ class CopickObject:
     def radius(self) -> Union[float, None]:
         return self.meta.radius
 
+    @property
+    def metadata(self) -> Dict[str, Any]:
+        return self.meta.metadata
+
     def zarr(self) -> Union[None, MutableMapping]:
         """Override this method to return a zarr store for this object. Should return None if
         CopickObject.is_particle is False or there is no associated map."""
@@ -448,11 +466,14 @@ class CopickRoot:
 
         return None
 
+    def _query_objects(self):
+        clz, meta_clz = self._object_factory()
+        self._objects = [clz(self, meta=obj) for obj in self.config.pickable_objects]
+
     @property
     def pickable_objects(self) -> List["CopickObject"]:
         if self._objects is None:
-            clz, meta_clz = self._object_factory()
-            self._objects = [clz(self, meta=obj) for obj in self.config.pickable_objects]
+            self._query_objects()
 
         return self._objects
 
@@ -474,6 +495,16 @@ class CopickRoot:
     def refresh(self) -> None:
         """Refresh the list of runs."""
         self._runs = self.query()
+        self._objects = None  # Reset objects to force reloading
+
+    def save_config(self, config_path: str) -> None:
+        """Save the configuration to a JSON file.
+
+        Args:
+            config_path: Path to the configuration file to save.
+        """
+        with open(config_path, "w") as f:
+            json.dump(self.config.model_dump(), f, indent=4)
 
     def new_run(self, name: str, exist_ok: bool = False, **kwargs) -> "CopickRun":
         """Create a new run.
@@ -528,6 +559,110 @@ class CopickRoot:
         """Override this method to return the run class and run metadata class."""
         return CopickRun, CopickRunMeta
 
+    def new_object(
+        self,
+        name: str,
+        is_particle: bool,
+        label: Optional[int] = None,
+        color: Optional[Tuple[int, int, int, int]] = None,
+        emdb_id: Optional[str] = None,
+        pdb_id: Optional[str] = None,
+        identifier: Optional[str] = None,
+        map_threshold: Optional[float] = None,
+        radius: Optional[float] = None,
+        metadata: Optional[Dict[str, Any]] = None,
+        exist_ok: bool = False,
+    ) -> "CopickObject":
+        """Create a new pickable object and add it to the configuration.
+
+        Args:
+            name: Name of the object.
+            is_particle: Whether this object should be represented by points (True) or segmentation masks (False).
+            label: Numeric label/id for the object. If None, will use the next available label.
+            color: RGBA color for the object. If None, will use a default color.
+            emdb_id: EMDB ID for the object.
+            pdb_id: PDB ID for the object.
+            identifier: Identifier for the object (e.g. Gene Ontology ID or UniProtKB accession).
+            map_threshold: Threshold to apply to the map when rendering the isosurface.
+            radius: Radius of the particle, when displaying as a sphere.
+            metadata: Additional metadata for the object (user-defined contents).
+            exist_ok: Whether existing objects with the same name should be overwritten..
+
+        Returns:
+            CopickObject: The newly created object.
+
+        Raises:
+            ValueError: If an object with the given name already exists and exist_ok is False.
+        """
+        sane_name = sanitize_name(name)
+
+        if name != sane_name:
+            raise ValueError(
+                f"Object name '{name}' contains invalid characters. Use copick.escape.sanitize_name() to clean it.",
+            )
+        name = sane_name
+
+        # Check if the object already exists
+        obj = self.get_object(name)
+        if obj and not exist_ok:
+            raise ValueError(f"Object name {name} already exists.")
+
+        if obj:
+            obj.meta.is_particle = is_particle
+            obj.meta.label = label if label else obj.label
+            obj.meta.color = color if color else obj.color
+            obj.meta.emdb_id = emdb_id if emdb_id else obj.emdb_id
+            obj.meta.pdb_id = pdb_id if pdb_id else obj.pdb_id
+            obj.meta.identifier = identifier if identifier else obj.identifier
+            obj.meta.map_threshold = map_threshold if map_threshold else obj.map_threshold
+            obj.meta.radius = radius if radius else obj.radius
+            obj.meta.metadata = metadata if metadata else obj.metadata
+        else:
+            # Check for duplicate label BEFORE auto-assignment
+            if label is not None:
+                existing_labels = {obj.label for obj in self.config.pickable_objects if obj.label is not None}
+
+                if label in existing_labels:
+                    raise ValueError(f"Object label {label} already exists.")
+
+            # Auto-assign label if not provided
+            if label is None:
+                existing_labels = [obj.label for obj in self.config.pickable_objects if obj.label is not None]
+                label = max(existing_labels) + 1 if existing_labels else 1
+
+            # Use default color if not provided
+            if color is None:
+                color = (100, 100, 100, 255)
+
+            # Create the pickable object metadata
+            pickable_meta = PickableObject(
+                name=name,
+                is_particle=is_particle,
+                label=label,
+                color=color,
+                emdb_id=emdb_id,
+                pdb_id=pdb_id,
+                identifier=identifier,
+                map_threshold=map_threshold,
+                radius=radius,
+                metadata=metadata if metadata else {},
+            )
+
+            # Add to configuration
+            self.config.pickable_objects.append(pickable_meta)
+
+            # Create the object and add to cache
+            clz, _ = self._object_factory()
+            obj = clz(self, pickable_meta, read_only=False)
+
+            # Ensure the objects cache is initialized before appending
+            if self._objects is None:
+                # This will initialize self._objects
+                _ = self.pickable_objects
+            self._objects.append(obj)
+
+        return obj
+
     def _object_factory(self) -> Tuple[Type["CopickObject"], Type["PickableObject"]]:
         """Override this method to return the object class and object metadata class."""
         return CopickObject, PickableObject
@@ -1057,6 +1192,9 @@ class CopickRun:
                 **kwargs,
             )
 
+            # Defer trimesh import to keep CLI snappy (trimesh imports scipy)
+            import trimesh
+
             # Need to create an empty trimesh.Trimesh object first, because empty scenes can't be exported.
             tmesh = trimesh.Trimesh()
             scene = tmesh.scene()
@@ -1971,7 +2109,7 @@ class CopickPicks:
 
         """
 
-        if positions.shape[0] != transforms.shape[0]:
+        if transforms is not None and positions.shape[0] != transforms.shape[0]:
             raise ValueError("Number of positions and transforms must be the same.")
 
         points = []
@@ -1985,6 +2123,20 @@ class CopickPicks:
         self.points = points
         self.store()
 
+    def df(self, format: str = "relion") -> "pd.DataFrame":
+        """Returns the points as a pandas DataFrame with columns based on the format."""
+        if format == "relion":
+            return picks_to_df_relion(self)
+        else:
+            raise ValueError(f"Format {format} is not supported.")
+
+    def from_df(self, df: "pd.DataFrame", format: str = "relion") -> None:
+        """Set the points from a pandas DataFrame with columns based on the format."""
+        if format == "relion":
+            relion_df_to_picks(self, df)
+        else:
+            raise ValueError(f"Format {format} is not supported.")
+
 
 class CopickMeshMeta(BaseModel):
     """Data model for mesh metadata.
@@ -2017,7 +2169,7 @@ class CopickMesh:
         color: Color of the pickable object this pick belongs to.
     """
 
-    def __init__(self, run: CopickRun, meta: CopickMeshMeta, mesh: Optional[Geometry] = None):
+    def __init__(self, run: CopickRun, meta: CopickMeshMeta, mesh: Optional["Geometry"] = None):
         self.meta: CopickMeshMeta = meta
         self.run: CopickRun = run
 
@@ -2049,7 +2201,7 @@ class CopickMesh:
     def color(self):
         return self.run.root.get_object(self.pickable_object_name).color
 
-    def _load(self) -> Geometry:
+    def _load(self) -> "Geometry":
         """Override this method to load mesh from a RESTful interface or filesystem."""
         raise NotImplementedError("load must be implemented for CopickMesh.")
 
@@ -2058,7 +2210,7 @@ class CopickMesh:
         the file if it doesn't exist."""
         raise NotImplementedError("store must be implemented for CopickMesh.")
 
-    def load(self) -> Geometry:
+    def load(self) -> "Geometry":
         """Load the mesh from storage.
 
         Returns:
@@ -2073,14 +2225,14 @@ class CopickMesh:
         self._store()
 
     @property
-    def mesh(self) -> Geometry:
+    def mesh(self) -> "Geometry":
         if self._mesh is None:
             self._mesh = self.load()
 
         return self._mesh
 
     @mesh.setter
-    def mesh(self, value: Geometry) -> None:
+    def mesh(self, value: "Geometry") -> None:
         self._mesh = value
 
     @property
